<?php
/**
 * Created by PhpStorm.
 * User: shensong
 * Date: 2022/10/13
 * Time: 14:18
 */

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Service\GoodsService;
use Illuminate\Http\Request;

class GoodsController extends Controller
{
    # 优选
    public function excellentEditor(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'min_id'  => 'nullable',
            'page_size' => 'required|numeric|in:1,2,10,20,50,100,200,500,1000',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $minId      = $request->input('min_id', 1);
        $pageSize   = $request->input('page_size', 20);

        $list = GoodsService::excellentEditor($minId, $pageSize);
        return self::returnValue($list);
    }

    # 好货推荐 今日搜榜
    public function salesList(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'min_id'    => 'nullable',
            'page_size' => 'required|numeric|in:1,2,10,20,50,100',
            'sale_type' => 'required|in:1,2,3,4'
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $minId      = $request->input('min_id', 1);
        $pageSize   = $request->input('page_size', 20);
        $saleType   = $request->input('sale_type');

        $list = GoodsService::salesList($minId, $pageSize, $saleType);
        return self::returnValue($list);
    }

    # 商品搜索
    public function goodsSearch(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'min_id'    => 'nullable',
            'tb_p'      => 'nullable',
            'page_no'   => 'nullable|numeric',
            'page_size' => 'required|numeric|in:1,2,10,20,50,100',
            'is_coupon' => 'nullable|in:0,1',
            'sort'      => 'required|in:1,2,3,4',
            'keyword'   => 'required',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $userId     = $request->input('user_id', 1);
        $minId      = $request->input('min_id', 1);
        $pageSize   = $request->input('page_size', 20);
        $keyword    = $request->input('keyword');
        $isCoupon   = $request->input('is_coupon', 0);
        $sort       = $request->input('sort', 1);
        $tbP        = $request->input('tb_p');
        $pageNo     = $request->input('page_no', 1);

        $list = GoodsService::goodsSearch($userId, $minId, $pageSize, $keyword, $isCoupon, $sort, $tbP, $pageNo);
        return self::returnValue($list);
    }

    # 商品详情
    public function goodsDetail(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'item_id'       => 'required',
            'promotion_id'  => 'required',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $itemId      = $request->input('item_id');
        $userId      = $request->input('user_id');
        $promotionId = $request->input('promotion_id');

        $data = GoodsService::goodsDetail($itemId, $promotionId, $userId);
        return self::returnValue($data);
    }

    # 相似商品
    public function sameGoods(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'item_id'    => 'required',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $itemId = $request->input('item_id');

        $data = GoodsService::sameGoods($itemId);
        return self::returnValue($data);
    }

    # 生成淘口令
    public function createTaobaoWord(Request $request)
    {
        $validator = \Validator::make($request->all(),[
            'item_id'       => 'required',
            'promotion_id'  => 'required',
            'item_title'    => 'required',
        ]);

        if ($validator->fails()) {
            return self::returnValue($validator->getMessageBag(), 1102);
        }

        $itemId     = $request->input('item_id');
        $promotionId= $request->input('promotion_id');
        $itemTitle  = $request->input('item_title');

        $data = GoodsService::createTaobaoWord($itemId, $promotionId, $itemTitle);
        return self::returnValue($data);
    }

    public function salesTypeList()
    {
        $list = GoodsService::salesTypeList();

        return self::returnValue($list);
    }
}
