<?php


namespace App\Service;


use App\Log;
use App\Support\EmailQueue;
use Illuminate\Support\Facades\Storage;

class FileUploadService
{
    /**
     * 上传素材
     * */
    public static function uploadFile($file)
    {
        try{
            if (empty($file) || !$file->isValid()) {
                return 1010;
            }

            $originalExtension = strtolower($file->getClientOriginalExtension());       //文件扩展名
            $realPath = $file->getRealPath();   //临时文件的绝对路径

            // 上传文件
            $filename = date('YmdHis') . '-' . uniqid() . '.' . $originalExtension;

            // 使用我们新建的uploads本地存储空间（目录）
            $result = Storage::disk('uploads')->put($filename, file_get_contents($realPath));

            if(!$result)
                return 1011;

            $oss = new OssService('kx-yhq');
            $localFilePath = '../storage/uploads/'.$filename;
            $ossFile = $oss->upload($originalExtension, $localFilePath, date("Y-m-d",time()));


            $fileUrl = $ossFile['oss-request-url'];
            @unlink($localFilePath);
        } catch (\Exception $e) {
            EmailQueue::rPush('素材上传过程发生异常', $e->getTraceAsString(), ['xiaohua.hou@kuxuan-inc.com'], '素材上传过程发生异常');
            Log::error('素材上传过程发生异常', [
                'line' => $e->getLine(),
                'msg' => $e->getMessage()
            ], 'UploadMaterial-Exception');

            return 1012;
        }

        return $fileUrl;
    }
}
