<?php


namespace App\Service;

use App\Models\Goods;
use App\Models\TopicGoods;
use App\Models\TopicInfo;

class TopicService
{
    /**
     * 获取专题列表
     * */
    public static function getTopicList($page, $pageSize)
    {
        list($list, $count) = TopicInfo::topicList($page, $pageSize);
        return [$list, $count];
    }

    /**
     * 获取专题商品
     * */
    public static function getTopicGoods($topicId, $page, $pageSize)
    {
        # 获取专题基本信息
        $topicInfo = TopicInfo::select('topic_name', 'topic_pic', 'topic_desc', 'pid')
            ->where('id', $topicId)->where('enable', 1)->first();
        if(empty($topicInfo)) return [[], 0];

        # 获取专题商品信息
        list($itemIds, $count) = TopicGoods::topicGoodsList($topicId, $page, $pageSize);
        if(!$count) {
            $topicInfo->goods_list = [];
            return [$topicInfo, $count];
        }

        # 通过接口获取商品信息
        $goodsList = Goods::getGoodsListByItemIds($itemIds);
        foreach ($goodsList as $goods) {
            $goods->coupon_money = $goods->coupon_money / 100;
            $goods->pid = $topicInfo->pid;
            # 商品主图显示像素
            $goods->item_pic = $goods->item_pic . '_310x310.jpg';

            # 商品原始价格
            $goods->item_price = $goods->zk_final_price ? $goods->zk_final_price / 100 : $goods->reserve_price / 100;
            # 商品最终价格
            $goods->item_end_price = round(($goods->item_price - $goods->coupon_money), 2);
            unset($goods->zk_final_price, $goods->reserve_price);
        }

        $topicInfo->goods_list = $goodsList;

        return [$topicInfo, $count];
    }
}
